"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var table_1 = require("primeng/components/table/table");
var moment = require("moment");
var base_filter_1 = require("../shared/base.filter");
/**
 * Clase base de los componentes TableView
 */
var BaseTableViewComponent = /** @class */ (function () {
    function BaseTableViewComponent(domainCachedService, tableViewService) {
        var _this = this;
        this.totalItems = 0;
        this.onPageChange = new core_1.EventEmitter();
        this.csvSeparator = ",";
        this.RESULTADOS_PAGE = base_filter_1.BaseFilter.RESULTADOS_PAGE; // se hace visible esta variable a cualquier subclase de tableview que pudiera necesitarla
        this.domainCachedService = domainCachedService;
        this.tableViewService = tableViewService;
        this.subscriptionMessages = tableViewService.observableExport.subscribe(function (message) {
            _this.exportCSV();
        });
    }
    BaseTableViewComponent.prototype.exportCSV = function () {
        var dt = this.dataTable;
        if (dt == null) {
            alert("Exportació no disponible");
            return;
        }
        dt.exportCSV();
    };
    BaseTableViewComponent.prototype.getDataTable = function () {
        return null;
    };
    BaseTableViewComponent.prototype.getColumna = function (record, field) {
        return null;
    };
    BaseTableViewComponent.prototype.getColumnaDominio = function (dominio, valor) {
        var domainMatch = dominio &&
            dominio.find(function (obj) {
                return obj.value == valor;
            });
        if (domainMatch) {
            return domainMatch.label;
        }
        else if (valor) {
            return String("??" + valor + "??");
        }
        return "";
    };
    BaseTableViewComponent.prototype.ngOnDestroy = function () {
        this.subscriptionMessages.unsubscribe();
    };
    BaseTableViewComponent.prototype.pageChanged = function (event) {
        console.log("pageChanged(" + event + ")");
        this.onPageChange.emit(event.page + 1);
    };
    /**
     * Método de ordenación de campos de fecha.
     * @see Adaptación de: https://stackoverflow.com/questions/41181277/primeng-datatable-custom-sorting-or-filtering-angular-2
     */
    BaseTableViewComponent.prototype.dateSort = function (event, field) {
        var comparer = function (a, b) {
            var formatedA = a[field] ? moment(a[field], "DD/MM/YYYY").format("YYYY-MM-DD") : "-";
            var formatedB = b[field] ? moment(b[field], "DD/MM/YYYY").format("YYYY-MM-DD") : "-";
            // let result = -1; if (moment(formatedB).isBefore(formatedA, 'day')) { result = 1; }
            var result = formatedA.localeCompare(formatedB);
            return result * event.order;
        };
        var strcmp = function (a, b) {
            if (a.toString() < b.toString()) {
                return -1;
            }
            if (a.toString() > b.toString()) {
                return 1;
            }
            return 0;
        };
        this.items.sort(comparer);
        this.items = this.items.slice();
    };
    /**Función de ordenación de columnas de los p-table de prime */
    BaseTableViewComponent.prototype.customColumnsSort = function (event, items) {
        var field = event.field;
        var vregexFecha = /^(0[1-9]|[1-2]\d|3[01])(\/)(0[1-9]|1[012])\2(\d{4})$/;
        var vregexFechaHora = /^(0[1-9]|[1-2]\d|3[01])(\/)(0[1-9]|1[012])\2(\d{4}) [012][0-9]:[0-6][0-9]$/;
        items.sort(function (a, b) {
            // comparamos DD/MM/YYYY
            if (vregexFecha.test(a[field]) && vregexFecha.test(b[field])) {
                var fechaA = a[field].substring(6) + a[field].substring(3, 5) + a[field].substring(0, 2);
                var fechaB = b[field].substring(6) + b[field].substring(3, 5) + b[field].substring(0, 2);
                var sortDesc = fechaA < fechaB ? (event.order === 1 ? 1 : -1) : 0;
                return fechaA > fechaB ? (event.order === 1 ? -1 : 1) : sortDesc;
            }
            // comparamos DD/MM/YYYY HH:MM
            else if (vregexFechaHora.test(a[field]) && vregexFechaHora.test(b[field])) {
                var fechaHoraA = a[field].substring(6, 10) +
                    a[field].substring(3, 5) +
                    a[field].substring(0, 2) +
                    a[field].substring(11, 13) +
                    a[field].substring(14);
                var fechaHoraB = b[field].substring(6, 10) +
                    b[field].substring(3, 5) +
                    b[field].substring(0, 2) +
                    b[field].substring(11, 13) +
                    b[field].substring(14);
                var sortDesc = fechaHoraA < fechaHoraB ? (event.order === 1 ? 1 : -1) : 0;
                return fechaHoraA > fechaHoraB ? (event.order === 1 ? -1 : 1) : sortDesc;
            }
            else {
                var sortDesc = a[field] < b[field] ? (event.order === 1 ? 1 : -1) : 0;
                return a[field] > b[field] ? (event.order === 1 ? -1 : 1) : sortDesc;
            }
        });
        return items.slice();
    };
    __decorate([
        core_1.ViewChild("dataTable", { static: false }),
        __metadata("design:type", table_1.Table)
    ], BaseTableViewComponent.prototype, "dataTable", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Number)
    ], BaseTableViewComponent.prototype, "totalItems", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], BaseTableViewComponent.prototype, "onPageChange", void 0);
    return BaseTableViewComponent;
}());
exports.BaseTableViewComponent = BaseTableViewComponent;
//# sourceMappingURL=base-tableview.component.js.map