"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var data_1 = require("@ngrx/data");
var operators_1 = require("rxjs/operators");
var ng2_file_upload_1 = require("ng2-file-upload");
var base_form_component_1 = require("./base-form.component");
var shared_1 = require("../../../shared");
/**
 * Componente base que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo T
 * y que además tendrá la posibilidad de subida de ficheros
 */
var BaseFormUploadComponent = /** @class */ (function (_super) {
    __extends(BaseFormUploadComponent, _super);
    function BaseFormUploadComponent(activatedRoute, messageService, formBuilder, location, actions$, entityService, dataService, changeDetector) {
        var _this = _super.call(this, activatedRoute, messageService, formBuilder, location, actions$, entityService) || this;
        _this.activatedRoute = activatedRoute;
        _this.messageService = messageService;
        _this.formBuilder = formBuilder;
        _this.location = location;
        _this.actions$ = actions$;
        _this.entityService = entityService;
        _this.dataService = dataService;
        _this.changeDetector = changeDetector;
        //parametros para personalizar el uploader
        _this.hasDropZone = true;
        _this.hasQueue = true;
        _this.hasSingleUpload = true;
        _this.hasMultipleUpload = true;
        _this.hasProgressBar = false; //inicialmente no hay barra de progreso. En el método setCanUpload podemos cambiar eso
        _this.deleteItemAfterUpload = true;
        _this.queueLimit = 10;
        _this.itemSizeLimit = 100000000;
        _this.sizeDropZone = 4;
        _this.sizeQueue = 8;
        _this.uploadedFiles = [];
        return _this;
    }
    BaseFormUploadComponent.prototype.ngOnInit = function () {
        _super.prototype.ngOnInit.call(this);
        this.createUploader();
        this.setCanUpload();
    };
    /** crea un uploader */
    BaseFormUploadComponent.prototype.createUploader = function () {
        this.uploader = new ng2_file_upload_1.FileUploader(this.getFileUploaderOptions());
    };
    /** options que definirán nuestro uploader */
    BaseFormUploadComponent.prototype.getFileUploaderOptions = function () {
        return {
            url: this.urlUpload,
            queueLimit: this.queueLimit,
            headers: [{ name: "Accept", value: "application/json" }],
            maxFileSize: this.itemSizeLimit,
            //authTokenHeader: "Authorization",  //si el método para subida al servidor está protegido podemos setear el token según corresponda
            //authToken: "Bearer " + token"
            disableMultipart: false // 'DisableMultipart' must be 'true' for formatDataFunction to be called.
            // formatDataFunctionIsAsync: true,
            // formatDataFunction: async item => {   //si no queremos utilizar la forma multipart si no enviar en el JSON todo el mega churro de bytes
            //   return new Promise((resolve, reject) => {
            //     resolve({
            //       name: item._file.name,
            //       length: item._file.size,
            //       contentType: item._file.type,
            //       date: new Date()
            //     });
            //   });
            // }
        };
    };
    /** Setea si el componente upload puede hacer uplodas individuales. Si no nos gusta este comportamiento podemos extender este método */
    BaseFormUploadComponent.prototype.setCanUpload = function () {
        if (this.isEditMode) {
            this.hasProgressBar = true;
            this.canUpload = true; //solo si estamos editando un objeto permitiemos subidas individuales
        }
    };
    /** Evento que se dispara dsde el uploader cuando un fichero individual ha sido subido al servidor de manera exitosa */
    BaseFormUploadComponent.prototype.onSuccess = function (event) {
        console.log(event);
        this.uploadedFiles.push(event.item);
        this.messageService.sendNotification(new shared_1.BitMessage("Subida fichero", "Fichero subido satisfactoriamente"));
    };
    /** Evento que se dispara dsde el uploader cuando un fichero individual NO ha sido subido al servidor de manera exitosa */
    BaseFormUploadComponent.prototype.onError = function (event) {
        var error = "error uploaded " + event.status;
        this.messageService.sendError(new shared_1.Error(event.item.url, event.response, event.response, error));
    };
    //sobreescribimos el método save estandar porque necesitamos setear los ficheros que puedan subirse
    BaseFormUploadComponent.prototype.save = function () {
        var _this = this;
        var fileExists = this.uploader.queue.length > 0;
        //primero averiguamos si hay ficheros para subir. En ese caso, mapeamos en el form
        if (!fileExists) {
            _super.prototype.save.call(this); //si no los tiene es un save normalito
        }
        else {
            //si hay ficheros a anexar vamos por otro lado
            //primero miramos los errores del formulario
            if (!this.myForm.valid) {
                var errores = shared_1.CustomValidators.validateForm(this.myForm);
                this.messageService.sendError(new shared_1.Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores));
            }
            else {
                //nos guardamos los ficheros a subir en filesToUpload
                var filesToUpload = new Array();
                for (var j = 0; j < this.uploader.queue.length; j++) {
                    var fileItem = this.uploader.queue[j]._file;
                    filesToUpload.push(fileItem);
                }
                //seteamos el object que irá al server. De momento sin los ficheros
                var newObject = __assign({}, this.object, this.myForm.value);
                if (this.preSave()) {
                    var service = //contendrá la función que nos hará el insert o el update. Es responsabilidad del componente que exitiende a este
                     void 0; //contendrá la función que nos hará el insert o el update. Es responsabilidad del componente que exitiende a este
                    //el proporcionarla implementando ambos métodos
                    if (this.formType == shared_1.FormType.Insert) {
                        service = this.insertAndUploadService(newObject, filesToUpload);
                    }
                    else {
                        service = this.updateAndUploadService(newObject, filesToUpload);
                    }
                    //cuando se ejecute la función de insert o update
                    service.pipe(operators_1.takeUntil(this.destroy$)).subscribe(function (result) {
                        _this.entityService.upsertOneInCache(result); //actualizamos la caché del entityData en el store
                        _this.entityService.createAndDispatch(data_1.EntityOp.COMMIT_ALL); //comitamos en el store los cambios
                        _this.postSave(result); //cosas a realizar tras guardar
                    });
                }
            }
        }
    };
    /** sobreescribimos el método postSave porque necesitamos resetear el campo de upload del template y además debemos resetear el uploader
     * con la opción para permitir subir ficheros de manera indiviual
     */
    BaseFormUploadComponent.prototype.postSave = function (result) {
        _super.prototype.postSave.call(this, result);
        this.setCanUpload();
        this.uploadComponent.removeQueue();
        this.uploader.setOptions(this.getFileUploaderOptions()); //reseteamos las opciones del FileUploader. Suele cambiar la URL de subida
        this.changeDetector.detectChanges(); //angular a veces es caprichoso con la detección de cambios y necesitamos avisarle que la haga
    };
    __decorate([
        core_1.ViewChild("upload", { static: false }),
        __metadata("design:type", shared_1.BitNGUploadComponent)
    ], BaseFormUploadComponent.prototype, "uploadComponent", void 0);
    return BaseFormUploadComponent;
}(base_form_component_1.BaseFormComponent));
exports.BaseFormUploadComponent = BaseFormUploadComponent;
//# sourceMappingURL=base-form-upload.component.js.map