import { OnInit, OnDestroy } from "@angular/core";
import { FormBuilder, FormGroup } from "@angular/forms";
import { ActivatedRoute } from "@angular/router";
import { Location } from "@angular/common";
import { Actions } from "@ngrx/effects";
import { EntityCollectionServiceBase } from "@ngrx/data";
import { Observable, Subject } from "rxjs";
import { PageStatus, FormType, MessageService } from "../../../shared";
import { BaseComponent } from "./base.component";
/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento de tipo T usando el framework
 * ngrx-data
 */
export declare abstract class BaseFormComponent<T> extends BaseComponent implements OnInit, OnDestroy {
    protected activatedRoute: ActivatedRoute;
    protected messageService: MessageService;
    protected formBuilder: FormBuilder;
    protected location: Location;
    protected actions$: Actions;
    protected entityService: EntityCollectionServiceBase<T>;
    myForm: FormGroup;
    object: T;
    status$: Observable<PageStatus>;
    toolbar$: Observable<{}>;
    breadcrumb: any[];
    formType: FormType;
    soloLectura: boolean;
    myTabs: any;
    initialTab: string;
    loadedTab: any;
    currentTab: number;
    destroy$: Subject<boolean>;
    constructor(activatedRoute: ActivatedRoute, messageService: MessageService, formBuilder: FormBuilder, location: Location, actions$: Actions, entityService: EntityCollectionServiceBase<T>);
    ngOnInit(): void;
    ngOnDestroy(): void;
    /** Primer valor de la toolbar al crear el objeto BaseForm */
    createToolbar(): Observable<{}>;
    /** Primer valor del objeto que se mapeará en el formulario asociado.  */
    abstract createObject(id: number): T;
    /** Construcción del FormGroup que tendrá asociado el formulario */
    abstract createFormGroup(): FormGroup;
    /** Valor del objeto breadcrumb dependiendo de si el componente va a insertar un nuevo objeto o actualizarlo */
    abstract createBreadcrumb(formType: FormType): any[];
    /** Establece el componente para editar un objeto ya existente */
    setModeUpdate(object: any): void;
    /** Establece el componente para insertar un nuevo objeto */
    setModeInsert(): void;
    /** Actualiza los valores del FormGroup con el valor actual almacenado en object */
    updateFormValues(): void;
    /** Actualiza el page status en función de tres observables. El estado de envío al servidor, el posible error producido y el éxito de la operación*/
    updatePageStatus(): void;
    /** Limpia el formulario. */
    abstract cleanForm(): void;
    /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
    readonly isEditMode: boolean;
    /**
     * Devuelve true si hay algún valor cambiado en la ficha.
     * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
     */
    readonly isDataChanged: boolean;
    /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
    readonly isFormValid: boolean;
    /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
    canDeactivate(): Observable<boolean> | boolean;
    /** Controla si el usuario sale de la página abrúptamente */
    unloadWithUnsavedChanges($event: any): string;
    /** Método que se dispara cuando algún botón de la toolbar es pulsado */
    onToolbarButtonPressed(button: string): void;
    /** Muestra la ayuda asociada a la pantalla. */
    help(): void;
    /** el componente toolbar nos comunica que se ha pulsado un botón */
    handlePrint(): void;
    /** contiene toda la lógica para enviar el formulario al servidor*/
    /** función útil que los componentes pueden extender para hacar algo antes de enviar el formulario al servidor */
    preSave(): boolean;
    save(): void;
    /** función para qué hacer después de insertar o modificar */
    postSave(object: any): void;
    preDelete(): void;
    /** contiene la lógica para eliminar el objeto representado en el formulario */
    delete(): void;
    postDelete(): void;
    /** Cualquier otra accion que no sea la típica de guardar o eliminar */
    otherAction(action: string): void;
    /** Cierra el formulario y vuelve a la pantalla anterior. */
    goBack(): void;
    isPreserveQueryParams(): boolean;
    /** Información de debug. */
    readonly diagnostic: string;
    /** función para hacer algo si queremos al cambiar de pestaña */
    changeTab(e: any): void;
    /** Registra una pestaña de visualización de información. */
    registerTab(index: number, id: string): void;
    initTabs(): void;
    /** Indica si la pestaña correspondiente es la inicial. */
    isInitialTab(tab: string): boolean;
    /** Indica si la pestaña correspondiente está cargada. */
    isLoadedTab(tab: string): any;
}
